"use client"

import { createCommissionExtra } from "@/actions/items/commissions/types/newType"
import SortableItem from "@/components/drag/SortableItem"
import { Button } from "@/components/ui/button"
import { DualRangeSlider } from "@/components/ui/dual-range"
import {
  FormControl,
  FormField,
  FormItem,
  FormLabel,
} from "@/components/ui/form"
import { Input } from "@/components/ui/input"
import { CommissionExtra } from "@/generated/prisma"
import {
  closestCenter,
  DndContext,
  DragEndEvent,
  PointerSensor,
  useSensor,
  useSensors,
} from "@dnd-kit/core"
import {
  SortableContext,
  verticalListSortingStrategy
} from "@dnd-kit/sortable"
import { useEffect, useState } from "react"
import { useFieldArray, useFormContext } from "react-hook-form"
import { ComboboxCreateable } from "./ComboboxCreateable"

type Props = {
  extras: CommissionExtra[]
}

export function CommissionExtraField({ extras: initialExtras }: Props) {
  const [mounted, setMounted] = useState(false)
  const { control } = useFormContext()
  const { fields, append, remove, move } = useFieldArray({
    control,
    name: "extras",
  })

  useEffect(() => {
    setMounted(true)
  }, [])

  const [extras, setExtras] = useState(initialExtras)

  const sensors = useSensors(useSensor(PointerSensor))

  const handleDragEnd = (event: DragEndEvent) => {
    const { active, over } = event

    if (!over || active.id === over.id) {
      return
    }

    const oldIndex = fields.findIndex((f) => f.id === active.id)
    const newIndex = fields.findIndex((f) => f.id === over.id)

    if (oldIndex !== -1 && newIndex !== -1) {
      move(oldIndex, newIndex)
    }
  }

  if (!mounted) return null

  return (
    <div className="space-y-4">
      <h3 className="text-lg font-semibold">Extras</h3>

      <DndContext sensors={sensors} collisionDetection={closestCenter} onDragEnd={handleDragEnd}>
        <SortableContext items={fields.map((f) => f.id)} strategy={verticalListSortingStrategy}>
          {fields.map((field, index) => (
            <SortableItem key={field.id} id={field.id}>
              <div className="grid grid-cols-5 gap-4 items-end">
                {/* Extra Picker (combobox with create) */}
                <FormField
                  control={control}
                  name={`extras.${index}.extraId`}
                  render={({ field: extraField }) => (
                    <FormItem>
                      <FormLabel>Extra</FormLabel>
                      <FormControl>
                        <ComboboxCreateable
                          options={extras.map((e) => ({
                            label: e.name,
                            value: e.id,
                          }))}
                          selected={extraField.value}
                          onSelect={extraField.onChange}
                          onCreateNew={async (name) => {
                            const newExtra = await createCommissionExtra({ name })
                            setExtras((prev) => [...prev, newExtra])
                            extraField.onChange(newExtra.id)
                          }}
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />

                {/* Price */}
                <FormField
                  control={control}
                  name={`extras.${index}.price`}
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>Price (€)</FormLabel>
                      <FormControl>
                        <Input
                          type="number"
                          step="0.01"
                          {...field}
                          value={field.value ?? ""}
                          onChange={(e) =>
                            field.onChange(e.target.value === "" ? undefined : e.target.valueAsNumber)
                          }
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />

                {/* Price Percent */}
                <FormField
                  control={control}
                  name={`extras.${index}.pricePercent`}
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>+ %</FormLabel>
                      <FormControl>
                        <Input
                          type="number"
                          step="0.01"
                          {...field}
                          value={field.value ?? ""}
                          onChange={(e) =>
                            field.onChange(e.target.value === "" ? undefined : e.target.valueAsNumber)
                          }
                        />
                      </FormControl>
                    </FormItem>
                  )}
                />

                {/* Range Slider */}
                <FormField
                  control={control}
                  name={`extras.${index}.priceRange`}
                  render={({ field }) => {
                    const [start, end] =
                      typeof field.value === "string" && field.value.includes("–")
                        ? field.value.split("–").map(Number)
                        : [0, 0]

                    return (
                      <FormItem>
                        <FormLabel>Range</FormLabel>
                        <FormControl>
                          <DualRangeSlider
                            value={[start, end]}
                            onChange={([min, max]) =>
                              field.onChange(`${Math.min(min, max)}–${Math.max(min, max)}`)
                            }
                          />
                        </FormControl>
                      </FormItem>
                    )
                  }}
                />

                {/* Remove Button */}
                <Button
                  type="button"
                  variant="destructive"
                  onClick={() => remove(index)}
                >
                  Remove
                </Button>
              </div>
            </SortableItem>
          ))}
        </SortableContext>
      </DndContext >

      <Button
        type="button"
        onClick={() =>
          append({
            extraId: "",
            price: undefined,
            pricePercent: undefined,
            priceRange: "0–0",
          })
        }
      >
        Add Extra
      </Button>
    </div >
  )
}
